/*jslint white:true, undef:true, nomen:true, eqeqeq:true, newcap:true */
/*global AJS, gadgets, console, document */

/**
 * @fileOverview The file implements a library of common gadget functions.
 */

// -----
// Namespacing
// -----

/**
 * @namespace Holds Atlassian JavaScript gadget resources.
 */
AJS.gadgets = AJS.gadgets || {};

// -----
// Data retrieval
// -----

/**
 * If true, the library will avoid resource caching by appending
 * a timestamp parameter to each GET request.
 */
AJS.gadgets.forceUncachedGet = false;

/**
 * Sends a GET request, with optional params, to the specified URL
 * and calls the callback function with the parsed JSON object.
 * Request errors will be logged to the console.
 * @param {String} baseUrl the URL to send the GET request to
 * @param {Object} params a mapping of GET param names to values
 * @param {Function} callback a function to receive the ready-to-go JSON
 */
AJS.gadgets.getJson = function (baseUrl, params, callback) {
    var paramArray = [];
    for (var param in params) {
        if (typeof param !== "function") {
            paramArray.push(param + "=" + params[param]);
        }
    }
    if (paramArray.length !== 0) {
        baseUrl = baseUrl + "?" + paramArray.join("&");
    }
    if (AJS.gadgets.forceUncachedGet) {
        if (paramArray.length !== 0) {
            baseUrl += "&";
        } else {
            baseUrl += "?";
        }
        baseUrl += "nocache=" + Date.now();
    }
    console.log("making json request: " + baseUrl);

    var preprocess = function (obj) {
        if (obj.rc !== 200) {
            var errorString = "";
            for (var prop in obj) {
                if (typeof obj[prop] !== "function") {
                    errorString += "\n" + prop + ": " + obj[prop];
                }
            }
            console.log("json load fail: " + errorString);
        }
        callback(obj);
    };

    var gParams = {};
    gParams[gadgets.io.RequestParameters.CONTENT_TYPE] = gadgets.io.ContentType.JSON;
    gadgets.io.makeRequest(baseUrl, preprocess, gParams);
};

/**
 * @namespace Helpers for making data requests using OAuth easier.
 */
AJS.gadgets.oauth = {};

/**
 * Convenience wrapper around the gadgets.io.makeRequest to make accessing data using OAuth easier.
 *
 * To fully utilize this function, there are a few things you need to do in your gadget.  First you need to require the
 * oauthpopup feature in you ModulePrefs and make sure to include your OAuth server provider information
 * 
 *   <?xml version="1.0" encoding="UTF-8" ?> 
 *   <Module>
 *   <ModulePrefs title="...">
 *      <Require feature="oauthpopup" />
 *      #oauth("http://oauth.gmodules.com/gadgets/oauthcallback")
 *   </ModulePrefs>
 * 
 * Next, you need to add a few elements to the gadget spec for the user to interact with during the OAuth dance, such
 * as a link for them to click on to personalize the gadget
 *     
 *   <div id="oauth-approval" style="display:none">
 *     <img src="http://gadget-doc-examples.googlecode.com/svn/trunk/images/new.gif">
 *     <a href="#" id="personalize-oauth-gadget">Personalize this gadget</a>
 *   </div>
 *
 * and an element that lets them know the authorization is under way and lets them indciate when the process is complete
 * 
 *   <div id="oauth-approval-done" style="display:none">
 *     Please click
 *     <a href="#">I've approved access</a>
 *     once you've approved access to your data.
 *   </div>
 *
 * Note: the text you use in the divs is unimportant, but the element IDs need to be exactly as they are here
 *
 * Finally, make the request for data:
 * 
 *   AJS.gadgets.oauth.makeRequest(url, {
 *     CONTENT_TYPE: gadgets.io.ContentType.TEXT,
 *     onDataAvailable: showResults,
 *     onOAuthError: showError
 *   });
 *   
 * In the above example we're making a request for text data.  If the user needs to do the OAuth dance, a 
 * "Personalize this gadget" link with element ID "personalize-oauth-gadget" is looked up and on onclick handler
 * attached.  When the user clicks on the personalize link, the oauthpopup feature is used to display the service
 * providers page asking the user for authorization.  When the popup is opened, the "Personalize this gadget" link is
 * hidden and a message indicating the approval is in process - with element ID "oauth-approval-done" - is displayed.
 * It should contain a link that the user can click to indicate the authorization process is done - normally they won't
 * have to do this as the popup should close on its own, but it may be necessary for the user to close the popup in
 * some situations.  Finally, once the user approves access and the popup is closed, the request for data is rerun with
 * the access token and the onDataAvailable function is called.  If there is any error during the OAuth dance, the 
 * onOAuthError function is called.
 * 
 * If you would rather handle the authorization yourself, you can do that by adding a function to the parameters object
 * named onApprovalRequired.  This will allow you to display your own popup or handle the authorization process in some
 * other manner.  It is passed the approval URL as a parameter.
 * 
 * If you want to change what is done when the user clicks on the personalize gadget link, then add a function
 * to the params object named onPopupOpen.
 * 
 * To change what happens when the popup closes, add the onPopupClose function to the params object.
 * 
 * @param {String} url the url to make a request to
 * @param params an object/array containing all the parameters to be passed to the gadgets.io.makeRequest function as
 * well as the following properties (optional unless noted otherwise)
 *   onDataAvailable(data): required function that is called when data is available from a request.
 *   onOAuthError(err, text): function that is called when an error occurs during the OAuth dance
 *   onApprovalRequired(approvalUrl): function that is called when the user needs to go through the OAuth dance
 *   onPopupOpen(): function called when the popup window is opened for approval (not called if onApprovalRequired is provided)  
 *   onPopupClose(): function called when the popup window is closed after approval (not called if onApprovalRequired is provided)  
 */
AJS.gadgets.oauth.makeRequest = function(url, params) {
    var doNothing = function() {};
    var showOAuthPopupLink = function(approvalUrl) {
        // Create the popup handler. The onPopupOpen function is called when the user
        // opens the popup window. The onPopupClose function is called when the popup
        // window is closed.
        var popup;
        var approvedOnClick;
        
        var onPopupOpen = function () {
            // The createApprovedOnClick function returns an onclick 
            // handler that will close the popup window and attempt to fetch the user's
            // data again.  It must be bound here so it can be unbound when the window closes, otherwise
            // click event handlers could pile up.
            approvedOnClick = popup.createApprovedOnClick();
            AJS.$('#oauth-approval-done a').bind("click", approvedOnClick);
            params.onPopupOpen();
        };
        var onPopupClose = function () {
            AJS.$('#oauth-approval-done a').unbind("click", approvedOnClick);
            params.onPopupClose();
        };
        popup = new gadgets.oauth.Popup(approvalUrl, null, onPopupOpen, onPopupClose);

        // Use the popup handler to attach onclick handlers to UI elements.  The
        // createOpenerOnClick() function returns an onclick handler to open the
        // popup window.
        AJS.$('#personalize-oauth-gadget').one("click", popup.createOpenerOnClick());
        AJS.$('#oauth-approval').show();
    };
    var showOAuthApprovalDoneLink = function() {
        AJS.$('#oauth-approval').hide();
        AJS.$('#oauth-approval-done').show();
    };
    var hideApprovalLinks = function() {
        AJS.$('#oauth-approval').hide();
        AJS.$('#oauth-approval-done').hide();
    };
    
    var gParams = params;
    if (gParams[gadgets.io.RequestParameters.AUTHORIZATION]
     && gParams[gadgets.io.RequestParameters.AUTHORIZATION] != gadgets.io.AuthorizationType.OAUTH) {
        console.log("Overriding gadgets.io.RequestParameters.AUTHORIZATION parameter with OAUTH");
    }
    gParams[gadgets.io.RequestParameters.AUTHORIZATION] = gadgets.io.AuthorizationType.OAUTH;

    params.onOAuthError = params.onOAuthError || doNothing;
    params.onApprovalRequired = params.onApprovalRequired || showOAuthPopupLink;
    params.onPopupOpen = params.onPopupOpen || showOAuthApprovalDoneLink;
    
    var makeRequest = function() {
        gadgets.io.makeRequest(url, function (response) { 
            if (response.oauthApprovalUrl) {
                params.onApprovalRequired(response.oauthApprovalUrl);
                if (response.data) {
                    params.onDataAvailable(response.data);
                }
            } else if (response.data) {
                hideApprovalLinks();
                params.onDataAvailable(response.data);
            } else if (response.oauthError) {
                hideApprovalLinks();
                params.onOAuthError(response.oauthError, response.oauthErrorText);
            }
        }, gParams);
    };
    params.onPopupClose = params.onPopupClose || function() {
        hideApprovalLinks();
        makeRequest();
    }
    makeRequest();
};
